@@ -1,7 +1,201 @@
+# Environment in which cookies will be stored. Cookies are expected to survive
+# the duration of the R session, but are not persisted outside of the R
+# session.
+.cookieStore <- new.env(parent=emptyenv())
+
+# Returns the cookies associated with a particular host/port
+# If no hostname is specified, returns all cookies
+getCookies <- function(hostname, port=NULL){
+  if (missing(hostname)){
+    hosts <- ls(envir=.cookieStore)
+    cookies <- lapply(hosts, function(h){
+      getCookiesHostname(h)
+    })
+    do.call("rbind", cookies)
+  } else {
+    host <- getCookieHost(list(host=hostname, port=port))
+    getCookiesHostname(host)
+  }
+}
+
+# Get cookies for a particular hostname(:port)
+getCookiesHostname <- function(host){
+  if (!exists(host, .cookieStore)){
+    NULL
+  } else {
+    cookies <- get(host, envir=.cookieStore)
+    cookies$host <- host
+    cookies
+  }
+}
+
+# Clears the cookies associated with a particular hostname/port combination.
+# If hostname and port are omitted, clears all the cookies
+clearCookies <- function(hostname, port=NULL){
+  if (missing(hostname)){
+    rm(list=ls(envir=.cookieStore), envir=.cookieStore)
+  } else {
+    host <- getCookieHost(list(host=hostname, port=port))
+    rm(list=host, envir=.cookieStore)
+  }
+}
+
 userAgent <- function() {
   paste("rsconnect", packageVersion("rsconnect"), sep="/")
 }
 
+getCookieHost <- function(requestURL){
+  host <- requestURL$host
+  port <- requestURL$port
+  if (!is.null(port) && nchar(port) > 0){
+    port <- sub("^:", "", port)
+    # By my reading of the RFC, we technically only need to include the port #
+    # in the index if the host is an IP address. But here we're including the
+    # port number as a part of the host whether using a domain name or IP.
+    # Erring on the side of not sending the cookies to the wrong services
+    host <- paste(host, port, sep=":")
+  }
+  host
+}
+
+# Parse out the raw headers provided and insert them into the cookieStore
+# NOTE: Domain attribute is currently ignored
+# @param requestURL the parsed URL as returned from `parseHttpUrl`
+# @param cookieHeaders a list of characters strings representing the raw
+#   Set-Cookie header value with the "Set-Cookie: " prefix omitted
+storeCookies <- function(requestURL, cookieHeaders){
+  cookies <- lapply(cookieHeaders, function(co){ parseCookie(requestURL, co) })
+
+  # Filter out invalid cookies (which would return as NULL)
+  cookies <- Filter(Negate(is.null), cookies)
+
+  host <- getCookieHost(requestURL)
+
+  hostCookies <- NULL
+  if (!exists(host, .cookieStore)){
+    # Create a new data frame for this host
+    hostCookies <- data.frame(
+      path=character(0),
+      name=character(0),
+      value=character(0),
+      secure=logical(0),
+      expires=character(0),
+      stringsAsFactors = FALSE
+    )
+  } else {
+    hostCookies <- get(host, envir=.cookieStore)
+  }
+
+  lapply(cookies, function(co){
+    # Remove any duplicates
+    # RFC says duplicate cookies are ones that have the same domain, name, and path
+    hostCookies <<- hostCookies[!(co$name == hostCookies$name & co$path == hostCookies$path),]
+
+    # append this new cookie on
+    hostCookies <<- rbind(as.data.frame(co, stringsAsFactors=FALSE), hostCookies)
+  })
+
+  # Save this host's cookies into the cookies store.
+  assign(host, hostCookies, envir=.cookieStore)
+}
+
+# Parse out an individual cookie
+# @param requestURL the parsed URL as returned from `parseHttpUrl`
+# @param cookieHeader the raw text contents of the Set-Cookie header with the
+#   header name omitted.
+parseCookie <- function(requestURL, cookieHeader){
+  keyval <- regmatches(cookieHeader, regexec(
+    "^(\\w+)\\s*=\\s*([^;]*)(;|$)", cookieHeader, ignore.case=TRUE))[[1]]
+  if (length(keyval) == 0){
+    # Invalid cookie format.
+    warning("Unable to parse set-cookie header: ", cookieHeader)
+    return(NULL)
+  }
+  key <- keyval[2]
+  val <- keyval[3]
+
+  # Path
+  path <- regmatches(cookieHeader, regexec(
+    "^.*\\sPath\\s*=\\s*([^;]+)(;|$).*$", cookieHeader, ignore.case=TRUE))[[1]]
+  if (length(path) == 0){
+    path <- "/"
+  } else {
+    path <- path[2]
+  }
+  if (!substring(requestURL$path, 1, nchar(path)) == path){
+    # Per the RFC, the cookie's path must be a prefix of the request URL
+    warning("Invalid path set for cookie on request for '", requestURL$path, "': ", cookieHeader)
+    return(NULL)
+  }
+
+  # MaxAge
+  maxage <- regmatches(cookieHeader, regexec(
+    "^.*\\sMax-Age\\s*=\\s*(-?\\d+)(;|$).*$", cookieHeader, ignore.case=TRUE))[[1]]
+  # If no maxage specified, then this is a session cookie, which means that
+  # (since our cookies only survive for a single session anyways...) we should
+  # keep this cookie around as long as we're alive.
+  expires <- Sys.time() + 10^10
+  if (length(maxage) > 0){
+    # Compute time maxage seconds from now
+    expires <- Sys.time() + as.numeric(maxage[2])
+  }
+
+  # Secure
+  secure <- grepl(";\\s+Secure(;|$)", cookieHeader, ignore.case=TRUE)
+
+  list(name=key,
+       value=val,
+       expires=expires,
+       path=path,
+       secure=secure)
+}
+
+# Appends a cookie header from the .cookieStore to the existing set of headers
+# @param requestURL the parsed URL as returned from `parseHttpUrl`
+# @param headers a named character vector containing the set of headers to be extended
+appendCookieHeaders <- function(requestURL, headers){
+  host <- getCookieHost(requestURL)
+
+  if (!exists(host, .cookieStore)){
+    # Nothing to do
+    return(headers)
+  }
+
+  cookies <- get(host, envir=.cookieStore)
+
+  # If any cookies are expired, remove them from the cookie store
+  if (any(cookies$expires < as.integer(Sys.time()))){
+    cookies <- cookies[cookies$expires >= as.integer(Sys.time()),]
+    # Update the store, removing the expired cookies
+    assign(host, cookies, envir=.cookieStore)
+  }
+
+  if (nrow(cookies) == 0){
+    # Short-circuit, return unmodified headers.
+    return(headers)
+  }
+
+  # Filter to only include cookies that match the path prefix
+  cookies <- cookies[substring(requestURL$path, 1, nchar(cookies$path)) == cookies$path,]
+
+  # If insecure channel, filter out secure cookies
+  if(tolower(requestURL$protocol) != "https"){
+    cookies <- cookies[!cookies$secure,]
+  }
+
+  # TODO: Technically per the RFC we're supposed to order these cookies by which
+  # paths most specifically match the request.
+  cookieHeader <- paste(apply(cookies, 1,
+                              function(x){ paste0(x["name"], "=", x["value"]) }), collapse="; ")
+
+  if (nrow(cookies) > 0){
+    return(c(headers, cookie=cookieHeader))
+  } else {
+    # Return unmodified headers
+    return(headers)
+  }
+}
+
 parseHttpUrl <- function(urlText) {
 
   matches <- regexec("(http|https)://([^:/#?]+)(?::(\\d+))?(.*)", urlText)
@@ -33,14 +227,17 @@ parseHttpStatusCode <- function(statusLine) {
     return (as.integer(statusCode))
 }
 
+# @param request a list containing protocol, host, port, method, and path fields
 readHttpResponse <- function(request, conn) {
   # read status code
   resp <- readLines(conn, 1)
   statusCode <- parseHttpStatusCode(resp[1])
 
   # read response headers
-  contentLength <- NULL
+  contentLength <- 0
+  contentType <- NULL
   location <- NULL
+  setCookies <- NULL
   repeat {
     resp <- readLines(conn, 1)
     if (nzchar(resp) == 0)
@@ -49,15 +246,21 @@ readHttpResponse <- function(request, conn) {
     header <- parseHttpHeader(resp)
     if (!is.null(header))
     {
-      if (identical(header$name, "Content-Type"))
+      name <- tolower(header$name)
+      if (name == "content-type")
         contentType <- header$value
-      if (identical(header$name, "Content-Length"))
+      if (name == "content-length")
         contentLength <- as.integer(header$value)
-      if (identical(header$name, "Location"))
+      if (name == "location")
         location <- header$value
+      if (name == "set-cookie")
+        setCookies <- c(setCookies, header$value)
     }
   }
 
+  # Store the cookies that were found in the request
+  storeCookies(request, setCookies)
+
   # read the response content
   content <- rawToChar(readBin(conn, what = 'raw', n=contentLength))
 
@@ -115,6 +318,8 @@ httpInternal <- function(protocol,
                                 "\r\n",
                                 sep=""))
   }
+  headers <- appendCookieHeaders(
+    list(protocol=protocol, host=host, port=port, path=path), headers)
   for (name in names(headers))
   {
     request <- c(request,
@@ -184,12 +389,16 @@ httpCurl <- function(protocol,
   if (!is.null(file))
     fileLength <- file.info(file)$size
 
+  headers <- appendCookieHeaders(
+    list(protocol=protocol, host=host, port=port, path=path), headers)
   extraHeaders <- character()
   for (header in names(headers))
   {
-    extraHeaders <- paste(extraHeaders, "--header")
-    extraHeaders <- paste(extraHeaders,
-                          paste('"', header,": ",headers[[header]], '"', sep=""))
+    if(!identical(header, "Content-Type") && !identical(header, "Content-Length")){
+      extraHeaders <- paste(extraHeaders, "--header")
+      extraHeaders <- paste(extraHeaders,
+                            paste('"', header,": ",headers[[header]], '"', sep=""))
+    }
   }
 
   outputFile <- tempfile()
@@ -224,7 +433,7 @@ httpCurl <- function(protocol,
                    "--silent",
                    "--show-error",
                    "-o", shQuote(outputFile),
-                   paste(protocol, "://", host, port, path, sep=""))
+                   paste('"', protocol, "://", host, port, path, '"', sep=""))
 
   result <- NULL
   time <- system.time(gcFirst = FALSE, {
@@ -318,6 +527,8 @@ httpRCurl <- function(protocol,
     options$verbose <- TRUE
 
   # add extra headers
+  headers <- appendCookieHeaders(
+    list(protocol=protocol, host=host, port=port, path=path), headers)
   extraHeaders <- as.character(headers)
   names(extraHeaders) <- names(headers)
   options$httpheader <- extraHeaders
@@ -378,6 +589,10 @@ httpRCurl <- function(protocol,
     "text/plain"
   }
 
+  # Parse cookies from header; bear in mind that there may be multiple headers
+  cookieHeaders <- headers[names(headers) == "Set-Cookie"]
+  storeCookies(list(protocol=protocol, host=host, port=port, path=path), cookieHeaders)
+
   contentValue <- textGatherer$value()
 
   # emit JSON trace if requested
